@@ -1,11 +1,11 @@
 /*
- * htmlimages.c
+ * Copyright (C) 1997-2003 Gary Wong <gtw@gnu.org>
+ * Copyright (C) 2002-2009 the AUTHORS
  *
- * by Gary Wong <gtw@gnu.org>, 1997-2002.
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 3 or later of the GNU General Public License as
- * published by the Free Software Foundation.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,10 +13,9 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
- * $Id: htmlimages.c,v 1.56 2017/04/02 20:01:23 plm Exp $
+ * $Id: htmlimages.c,v 1.60 2022/02/21 22:15:02 plm Exp $
  */
 
 #include "config.h"
@@ -72,8 +71,8 @@ static unsigned char *auchMidlb;
 static void
 WriteImageStride(unsigned char *img, int stride, int cx, int cy)
 {
-    if (WritePNG(szFile, img, stride, cx, cy) == -1)
-        outputf("Error creating file %s\n", szFile);
+    if (WritePNG(szFile, img, (unsigned int) stride, (unsigned int) cx, (unsigned int) cy) == -1)
+        outputf(_("Error creating image file %s\n"), szFile);
     imagesWritten++;
     ProgressValueAdd(1);
 }
@@ -566,7 +565,7 @@ WriteImages(void)
     }
 
     if (imagesWritten != NUM_IMAGES)
-        g_print("Wrong number of images generated - %d written, expected %d\n", imagesWritten, NUM_IMAGES);
+        g_print(_("Wrong number of images generated - %d written, expected %d\n"), imagesWritten, NUM_IMAGES);
 }
 
 static void
@@ -616,15 +615,16 @@ GetFilenameBase(char *sz)
 
     if (!sz || !*sz) {
         outputf(_("You must specify a directory to export to (see `%s')\n"), "help export htmlimages");
-        return 0;
+        return NULL;
     }
 
     if (!g_file_test(sz, G_FILE_TEST_IS_DIR) && g_mkdir(sz, 0777) < 0) {
         outputerr(sz);
-        return 0;
+        return NULL;
     }
 
-    szFile = malloc(strlen(sz) + 32);
+    szFile = g_malloc(strlen(sz) + 32);
+
     strcpy(szFile, sz);
 
     if (szFile[strlen(szFile) - 1] != '/')
@@ -685,7 +685,7 @@ TidyObjects(void)
     free(auchArrow[0]);
     free(auchArrow[1]);
 #endif
-    free(szFile);
+    g_free(szFile);
     free(auchBoard);
     for (i = 0; i < 2; i++)
         free(auchChequer[i]);
@@ -718,15 +718,13 @@ extern void
 CommandExportHTMLImages(char *sz)
 {
     szFile = GetFilenameBase(sz);
-    if (!szFile)
-        return;
 
     ProgressStartValue(_("Generating image:"), NUM_IMAGES);
     AllocObjects();
     RenderObjects();
     WriteImages();
-    ProgressEnd();
     TidyObjects();
+    ProgressEnd();
 }
 
 #else                           /* not HAVE_LIBPNG */
