@@ -1,11 +1,11 @@
 /*
- * progress.c
+ * Copyright (C) 2003 Joern Thyssen <jth@gnubg.org>
+ * Copyright (C) 2003-2018 the AUTHORS
  *
- * by Joern Thyssen <jth@gnubg.org>, 2003
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 3 or later of the GNU General Public License as
- * published by the Free Software Foundation.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,10 +13,9 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
- * $Id: progress.c,v 1.71 2013/09/05 18:43:46 plm Exp $
+ * $Id: progress.c,v 1.86 2022/01/09 14:09:14 plm Exp $
  */
 
 #include "config.h"
@@ -26,40 +25,32 @@
 #include <stdlib.h>
 #include <glib.h>
 #include <math.h>
-
-#if USE_GTK
-#include <gtk/gtk.h>
-#endif                          /* USE_GTK */
-
+#include <time.h>
 
 #include "eval.h"
 #include "rollout.h"
 #include "progress.h"
 #include "backgammon.h"
 #include "format.h"
-#include "time.h"
 
-#if USE_GTK
+#if defined(USE_GTK)
 #include "gtkgame.h"
 #include "gtkwindows.h"
-typedef enum _rollout_colls {
+typedef enum {
     TITLE_C, RANK_C, TRIALS_C, WIN_C, WIN_G_C, WIN_BG_C, LOSE_G_C, LOSE_BG_C, CLESS_C, CFUL_C, CFUL_S_C, JSD_C,
     N_ROLLOUT_COLS
 } rollout_cols;
-
 #endif                          /* USE_GTK */
 
 
-typedef struct _rolloutprogress {
-
+typedef struct {
     void *p;
     int n;
     char **ppch;
     int iNextAlternative;
     int iNextGame;
     time_t tStart;
-
-#if USE_GTK
+#if defined(USE_GTK)
     rolloutstat *prs;
     GtkWidget *pwRolloutDialog;
     GtkWidget *pwRolloutResult;
@@ -77,21 +68,20 @@ typedef struct _rolloutprogress {
     char ***pListText;
     int stopped;
 #endif
-
 } rolloutprogress;
 
-#if USE_GTK
+#if defined(USE_GTK)
 static void
 AllocTextList(rolloutprogress * prp)
 {                               /* 2d array to cache displayed widget text */
     int i;
     int lines = prp->n;
-    prp->pListText = malloc(sizeof(char **) * lines * 2);
+    prp->pListText = g_malloc(sizeof(char **) * lines * 2);
 
     for (i = 0; i < lines; i++) {
-        prp->pListText[i * 2] = malloc(sizeof(char *) * (N_ROLLOUT_COLS));
+        prp->pListText[i * 2] = g_malloc(sizeof(char *) * (N_ROLLOUT_COLS));
         memset(prp->pListText[i * 2], 0, sizeof(char *) * (N_ROLLOUT_COLS));
-        prp->pListText[i * 2 + 1] = malloc(sizeof(char *) * (N_ROLLOUT_COLS));
+        prp->pListText[i * 2 + 1] = g_malloc(sizeof(char *) * (N_ROLLOUT_COLS));
         memset(prp->pListText[i * 2 + 1], 0, sizeof(char *) * (N_ROLLOUT_COLS));
     }
 }
@@ -103,27 +93,21 @@ FreeTextList(rolloutprogress * prp)
     int lines = prp->n;
 
     for (i = 0; i < lines; i++) {
-        free(prp->pListText[i * 2]);
-        free(prp->pListText[i * 2 + 1]);
+        g_free(prp->pListText[i * 2]);
+        g_free(prp->pListText[i * 2 + 1]);
     }
-    free(prp->pListText);
+    g_free(prp->pListText);
 }
-
 #endif
 
-
-/*
- *
- */
-
 static time_t
 time_left(unsigned int n_games_todo, unsigned int n_games_done, unsigned int initial_game_count, time_t t_start)
 {
     float pt;
     time_t t_now;
     time(&t_now);
-    pt = ((float) n_games_todo) / (n_games_done - initial_game_count);
-    return (time_t) (pt * (t_now - t_start));
+    pt = (float) n_games_todo / (float) (n_games_done - initial_game_count);
+    return (time_t) (pt * (float) (t_now - t_start));
 }
 
 static char *
@@ -151,13 +135,11 @@ static float
 estimatedSE(const float rSE, const int iGame, const int nTrials)
 {
 
-    return rSE * (float) sqrt((1.0 * iGame) / (1.0 * (nTrials - 1)));
-
+    return rSE * sqrtf((float) iGame / (float) (nTrials - 1));
 
 }
 
-#if USE_GTK
-
+#if defined(USE_GTK)
 /*
  * Make pages with statistics.
  */
@@ -176,7 +158,7 @@ add_stat_columns(GtkTreeView * treeview, const char *title, const char **headers
 
     for (i = 0; i < 2; i++) {
         for (j = 0; j < n; j++) {
-            char *header = g_strdup_printf("%s%s", headers[j], ap[i].szName);
+            char *header = g_strdup_printf("%s%s", (strlen(headers[j]) > 0) ? gettext(headers[j]) : "", ap[i].szName);
             renderer = gtk_cell_renderer_text_new();
             g_object_set(renderer, "xalign", 1.0, NULL);
             column = gtk_tree_view_column_new_with_attributes(header, renderer, "text", 1 + j + i * n, NULL);
@@ -234,11 +216,11 @@ create_win_model(const rolloutstat * prs, int cGames, int *cGamesCount)
         g_free(s[j]);
 
     for (j = 0; j < 6; j++) {
-        s[j] = g_strdup_printf("%6.2f%%", 100.0 * (float) anTotal[j] / (float) cGames);
+        s[j] = g_strdup_printf("%6.2f%%", 100.0f * (float) anTotal[j] / (float) cGames);
         *cGamesCount += anTotal[j];
     }
     gtk_list_store_append(store, &iter);
-    gtk_list_store_set(store, &iter, 0, "%%", 1, s[0], 2, s[1], 3, s[2], 4, s[3], 5, s[4], 6, s[5], -1);
+    gtk_list_store_set(store, &iter, 0, "", 1, s[0], 2, s[1], 3, s[2], 4, s[3], 5, s[4], 6, s[5], -1);
     for (j = 0; j < 6; j++)
         g_free(s[j]);
 
@@ -256,7 +238,11 @@ GTKStatPageWin(const rolloutstat * prs, const int cGames)
     int cGamesCount = 0;
     char *sz;
 
+#if GTK_CHECK_VERSION(3,0,0)
+    pw = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
+#else
     pw = gtk_vbox_new(FALSE, 0);
+#endif
 
     pwLabel = gtk_label_new(_("Win statistics"));
 
@@ -270,7 +256,17 @@ GTKStatPageWin(const rolloutstat * prs, const int cGames)
 
     gtk_box_pack_start(GTK_BOX(pw), treeview, TRUE, TRUE, 0);
 
-    sz = g_strdup_printf(_("%d/%d games truncated"), (cGames - cGamesCount), cGames);
+    /* Truncated should be the number of games truncated at the
+     * 2-sided bearoff database, but there is another possible
+     * discrepancy between cGames and cGamesCount for cube rollouts :
+     * one of the double/no-double branch could have had up to
+     * nthreads less trials than the other, leading to a negative
+     * number here if there is no truncation, like in match play.
+     * Don't display this but a 0 instead.
+     */
+    sz = g_strdup_printf(_("%d/%d games truncated"), ((cGames - cGamesCount) >= 0 ? (cGames - cGamesCount) : 0),
+                         cGames);
+
     pwLabel = gtk_label_new(sz);
     gtk_box_pack_start(GTK_BOX(pw), pwLabel, FALSE, FALSE, 4);
     g_free(sz);
@@ -317,9 +313,9 @@ create_cube_model(const rolloutstat * prs, int cGames, int *anTotal)
         g_free(s[j]);
 
     for (j = 0; j < 4; j++)
-        s[j] = g_strdup_printf("%6.2f%%", 100.0 * (float) anTotal[j] / (float) cGames);
+        s[j] = g_strdup_printf("%6.2f%%", 100.0f * (float) anTotal[j] / (float) cGames);
     gtk_list_store_append(store, &iter);
-    gtk_list_store_set(store, &iter, 0, "%%", 1, s[0], 2, s[1], 3, s[2], 4, s[3], -1);
+    gtk_list_store_set(store, &iter, 0, "", 1, s[0], 2, s[1], 3, s[2], 4, s[3], -1);
     for (j = 0; j < 4; j++)
         g_free(s[j]);
 
@@ -337,7 +333,11 @@ GTKStatPageCube(const rolloutstat * prs, const int cGames)
     int anTotal[4];
     char sz[100];
 
+#if GTK_CHECK_VERSION(3,0,0)
+    pw = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
+#else
     pw = gtk_vbox_new(FALSE, 0);
+#endif
 
     pwLabel = gtk_label_new(_("Cube statistics"));
 
@@ -352,13 +352,15 @@ GTKStatPageCube(const rolloutstat * prs, const int cGames)
     gtk_box_pack_start(GTK_BOX(pw), treeview, TRUE, TRUE, 0);
 
     if (anTotal[1] + anTotal[0]) {
-        sprintf(sz, _("Cube efficiency for %s: %7.4f"), ap[0].szName, (float) anTotal[0] / (anTotal[1] + anTotal[0]));
+        sprintf(sz, _("Cube efficiency for %s: %7.4f"), ap[0].szName,
+                (float) anTotal[0] / (float) (anTotal[1] + anTotal[0]));
         pwLabel = gtk_label_new(sz);
         gtk_box_pack_start(GTK_BOX(pw), pwLabel, FALSE, FALSE, 4);
     }
 
     if (anTotal[2] + anTotal[3]) {
-        sprintf(sz, _("Cube efficiency for %s: %7.4f"), ap[1].szName, (float) anTotal[2] / (anTotal[3] + anTotal[2]));
+        sprintf(sz, _("Cube efficiency for %s: %7.4f"), ap[1].szName,
+                (float) anTotal[2] / (float) (anTotal[3] + anTotal[2]));
         pwLabel = gtk_label_new(sz);
         gtk_box_pack_start(GTK_BOX(pw), pwLabel, FALSE, FALSE, 4);
     }
@@ -389,12 +391,12 @@ create_bearoff_model(const rolloutstat * prs)
     g_free(s2);
 
     if (prs->nBearoffMoves)
-        s1 = g_strdup_printf("%7.2f", (float) prs->nBearoffPipsLost / prs->nBearoffMoves);
+        s1 = g_strdup_printf("%7.2f", (float) prs->nBearoffPipsLost / (float) prs->nBearoffMoves);
     else
         s1 = g_strdup_printf("n/a");
 
     if ((prs + 1)->nBearoffMoves)
-        s2 = g_strdup_printf("%7.2f", (float) (prs + 1)->nBearoffPipsLost / (prs + 1)->nBearoffMoves);
+        s2 = g_strdup_printf("%7.2f", (float) (prs + 1)->nBearoffPipsLost / (float) (prs + 1)->nBearoffMoves);
     else
         s2 = g_strdup_printf("n/a");
     gtk_list_store_append(store, &iter);
@@ -414,7 +416,11 @@ GTKStatPageBearoff(const rolloutstat * prs, const int UNUSED(cGames))
     GtkTreeModel *model;
     const char *headers[] = { "" };
 
+#if GTK_CHECK_VERSION(3,0,0)
+    pw = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
+#else
     pw = gtk_vbox_new(FALSE, 0);
+#endif
 
     pwLabel = gtk_label_new(_("Bearoff statistics"));
 
@@ -449,11 +455,12 @@ create_closed_out_model(const rolloutstat * prs)
     g_free(s2);
 
     if (prs->nOpponentClosedOut)
-        s1 = g_strdup_printf("%7.2f", 1.0f + prs->rOpponentClosedOutMove / prs->nOpponentClosedOut);
+        s1 = g_strdup_printf("%7.2f", 1.0f + (float) prs->rOpponentClosedOutMove / (float) prs->nOpponentClosedOut);
     else
         s1 = g_strdup("n/a");
     if ((prs + 1)->nOpponentClosedOut)
-        s2 = g_strdup_printf("%7.2f", 1.0f + (prs + 1)->rOpponentClosedOutMove / (prs + 1)->nOpponentClosedOut);
+        s2 = g_strdup_printf("%7.2f",
+                             1.0f + (float) (prs + 1)->rOpponentClosedOutMove / (float) (prs + 1)->nOpponentClosedOut);
     else
         s2 = g_strdup("n/a");
     gtk_list_store_append(store, &iter);
@@ -473,7 +480,11 @@ GTKStatPageClosedOut(const rolloutstat * prs, const int UNUSED(cGames))
     GtkTreeModel *model;
     const char *headers[] = { "" };
 
+#if GTK_CHECK_VERSION(3,0,0)
+    pw = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
+#else
     pw = gtk_vbox_new(FALSE, 0);
+#endif
 
     pwLabel = gtk_label_new(_("Closed out statistics"));
 
@@ -514,12 +525,12 @@ create_hit_model(const rolloutstat * prs, int cGames)
     g_free(s2);
 
     if (prs->nOpponentHit)
-        s1 = g_strdup_printf("%7.2f", 1.0f + prs->rOpponentHitMove / (1.0 * prs->nOpponentHit));
+        s1 = g_strdup_printf("%7.2f", 1.0f + (float) prs->rOpponentHitMove / (float) prs->nOpponentHit);
     else
         s1 = g_strdup_printf("n/a");
 
     if ((prs + 1)->nOpponentHit)
-        s2 = g_strdup_printf("%7.2f", 1.0f + (prs + 1)->rOpponentHitMove / (1.0 * (prs + 1)->nOpponentHit));
+        s2 = g_strdup_printf("%7.2f", 1.0f + (float) (prs + 1)->rOpponentHitMove / (float) (prs + 1)->nOpponentHit);
     else
         s2 = g_strdup_printf("n/a");
     gtk_list_store_append(store, &iter);
@@ -539,7 +550,11 @@ GTKStatPageHit(const rolloutstat * prs, const int cGames)
     GtkTreeModel *model;
     const char *headers[] = { "" };
 
+#if GTK_CHECK_VERSION(3,0,0)
+    pw = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
+#else
     pw = gtk_vbox_new(FALSE, 0);
+#endif
 
     pwLabel = gtk_label_new(_("Hit statistics"));
 
@@ -560,8 +575,6 @@ GTKStatPageHit(const rolloutstat * prs, const int cGames)
 static GtkWidget *
 GTKRolloutStatPage(const rolloutstat * prs, const int cGames)
 {
-
-
     /* GTK Widgets */
 
     GtkWidget *pw;
@@ -571,7 +584,11 @@ GTKRolloutStatPage(const rolloutstat * prs, const int cGames)
 
     /* Create notebook pages */
 
+#if GTK_CHECK_VERSION(3,0,0)
+    pw = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
+#else
     pw = gtk_vbox_new(FALSE, 0);
+#endif
 
     pwWin = GTKStatPageWin(prs, cGames);
     pwCube = GTKStatPageCube(prs, cGames);
@@ -589,11 +606,13 @@ GTKRolloutStatPage(const rolloutstat * prs, const int cGames)
     gtk_box_pack_start(GTK_BOX(pw), pwClosedOut, FALSE, FALSE, 0);
     gtk_box_pack_start(GTK_BOX(pw), pwHit, FALSE, FALSE, 0);
 
+#if GTK_CHECK_VERSION(3, 8, 0)
+    gtk_container_add(GTK_CONTAINER(psw), pw);
+#else
     gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(psw), pw);
-
+#endif
 
     return psw;
-
 }
 
 
@@ -643,7 +662,7 @@ GTKViewRolloutStatistics(GtkWidget * UNUSED(widget), gpointer data)
 }
 
 static void
-RolloutCancel(GtkObject * UNUSED(po), rolloutprogress * prp)
+RolloutCancel(GObject * UNUSED(po), rolloutprogress * prp)
 {
     pwGrab = pwOldGrab;
     prp->pwRolloutDialog = NULL;
@@ -654,7 +673,7 @@ RolloutCancel(GtkObject * UNUSED(po), rolloutprogress * prp)
 }
 
 static void
-RolloutStop(GtkObject * UNUSED(po), rolloutprogress * prp)
+RolloutStop(GObject * UNUSED(po), rolloutprogress * prp)
 {
     fInterrupt = TRUE;
     prp->stopped = -1;
@@ -662,14 +681,14 @@ RolloutStop(GtkObject * UNUSED(po), rolloutprogress * prp)
 
 
 static void
-RolloutStopAll(GtkObject * UNUSED(po), rolloutprogress * prp)
+RolloutStopAll(GObject * UNUSED(po), rolloutprogress * prp)
 {
     fInterrupt = TRUE;
     prp->stopped = -2;
 }
 
 static void
-create_rollout_list(int n, char asz[][40], GtkWidget ** View, GtkListStore ** List, gboolean cubeful)
+create_rollout_list(int n, char asz[][FORMATEDMOVESIZE], GtkWidget ** View, GtkListStore ** List, gboolean cubeful)
 {
     int i;
     GtkTreeModel *sort_model;
@@ -684,10 +703,10 @@ create_rollout_list(int n, char asz[][40], GtkWidget ** View, GtkListStore ** Li
         N_("Lose (bg)"),
         N_("Cubeless"),
         N_("Cubeful"),
-        N_("Std dev"),
+        N_("Std Err"),
         N_("JSDs")
     };
-    char *aszTemp[N_ROLLOUT_COLS];
+    const char *aszTemp[N_ROLLOUT_COLS];
 
     for (i = 0; i < N_ROLLOUT_COLS; i++)
         aszTemp[i] = aszTitle[i] ? gettext(aszTitle[i]) : "";
@@ -719,6 +738,8 @@ create_rollout_list(int n, char asz[][40], GtkWidget ** View, GtkListStore ** Li
         renderer = gtk_cell_renderer_text_new();
         column = gtk_tree_view_column_new_with_attributes(aszTemp[i], renderer, "text", i, NULL);
         gtk_tree_view_append_column(GTK_TREE_VIEW(*View), column);
+        if (i == 0)
+            gtk_tree_view_column_set_expand(column, TRUE);
         gtk_tree_view_column_set_sort_column_id(column, i);
     }
     gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(sort_model), RANK_C, GTK_SORT_ASCENDING);
@@ -726,7 +747,8 @@ create_rollout_list(int n, char asz[][40], GtkWidget ** View, GtkListStore ** Li
 
 static void
 GTKRolloutProgressStart(const cubeinfo * UNUSED(pci), const int n,
-                        rolloutstat aars[][2], rolloutcontext * prc, char asz[][40], gboolean multiple, void **pp)
+                        rolloutstat aars[][2], rolloutcontext * prc, char asz[][FORMATEDMOVESIZE], gboolean multiple,
+                        void **pp)
 {
 
     gchar *sz;
@@ -736,7 +758,10 @@ GTKRolloutProgressStart(const cubeinfo * UNUSED(pci), const int n,
     rolloutprogress *prp = (rolloutprogress *) g_malloc(sizeof(rolloutprogress));
     *pp = prp;
     prp->prs = (rolloutstat *) aars;
+    if (aars)
+        memset(aars, 0, 2 * n * sizeof(rolloutstat));
     prp->n = n;
+    prp->nGamesDone = 0;
     prp->stopped = 0;
     fInterrupt = FALSE;
 
@@ -780,18 +805,29 @@ GTKRolloutProgressStart(const cubeinfo * UNUSED(pci), const int n,
 
     g_signal_connect(G_OBJECT(prp->pwRolloutViewStat), "clicked", G_CALLBACK(GTKViewRolloutStatistics), prp);
 
+#if GTK_CHECK_VERSION(3,0,0)
+    pwVbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 4);
+#else
     pwVbox = gtk_vbox_new(FALSE, 4);
+#endif
+
     create_rollout_list(n, asz, &prp->pwRolloutResult, &prp->pwRolloutResultList, prc->fCubeful);
     prp->pwRolloutProgress = gtk_progress_bar_new();
+#if GTK_CHECK_VERSION(3,0,0)
+    gtk_progress_bar_set_show_text(GTK_PROGRESS_BAR(prp->pwRolloutProgress), TRUE);
+    gtk_progress_bar_set_text(GTK_PROGRESS_BAR(prp->pwRolloutProgress), "");
+#endif
 
     gtk_box_pack_start(GTK_BOX(pwVbox), prp->pwRolloutResult, TRUE, TRUE, 0);
     gtk_box_pack_start(GTK_BOX(pwVbox), prp->pwRolloutProgress, FALSE, FALSE, 0);
 
-
-
     /* time elapsed and left */
 
+#if GTK_CHECK_VERSION(3,0,0)
+    pwhbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 4);
+#else
     pwhbox = gtk_hbox_new(FALSE, 4);
+#endif
     gtk_box_pack_start(GTK_BOX(pwVbox), pwhbox, FALSE, FALSE, 0);
 
     gtk_box_pack_start(GTK_BOX(pwhbox), gtk_label_new(_("Time elapsed")), FALSE, FALSE, 4);
@@ -799,9 +835,9 @@ GTKRolloutProgressStart(const cubeinfo * UNUSED(pci), const int n,
     gtk_box_pack_start(GTK_BOX(pwhbox), gtk_label_new(_("Estimated time left")), FALSE, FALSE, 4);
     gtk_box_pack_start(GTK_BOX(pwhbox), prp->pwLeft = gtk_label_new(_("n/a")), FALSE, FALSE, 4);
     if (asz && asz[0] && *asz[0])
-        sz = g_strdup_printf(_("Estimated SE for \"%s\" after %d trials "), asz[0], prc->nTrials);
+        sz = g_strdup_printf(_("Estimated SE for \"%s\" after %u trials "), asz[0], prc->nTrials);
     else
-        sz = g_strdup_printf(_("Estimated SE after %d trials "), prc->nTrials);
+        sz = g_strdup_printf(_("Estimated SE after %u trials "), prc->nTrials);
 
     gtk_box_pack_start(GTK_BOX(pwhbox), gtk_label_new(sz), FALSE, FALSE, 4);
     g_free(sz);
@@ -832,8 +868,6 @@ GTKRolloutProgress(float aarOutput[][NUM_ROLLOUT_OUTPUTS],
     static int min_games_done = 0;
     char sz[32];
     int i;
-    gchar *gsz;
-    double frac;
     GtkTreeIter iter;
 
     if (!prp || !prp->pwRolloutResult)
@@ -882,8 +916,9 @@ GTKRolloutProgress(float aarOutput[][NUM_ROLLOUT_OUTPUTS],
             min_games_done = iGame + 1;
     }
     if (iAlternative == (prp->n - 1)) {
-        frac = ((float) min_games_done) / prc->nTrials;
-        gsz = g_strdup_printf("%d/%d (%d%%)", min_games_done, prc->nTrials, (int) (100.0f * frac));
+        float frac = (float) min_games_done / (float) prc->nTrials;
+        gchar *gsz = g_strdup_printf("%d/%u (%d%%)", min_games_done, prc->nTrials, (int) (100.0f * frac));
+
         gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(prp->pwRolloutProgress), frac);
         gtk_progress_bar_set_text(GTK_PROGRESS_BAR(prp->pwRolloutProgress), gsz);
         g_free(gsz);
@@ -960,14 +995,12 @@ GTKRolloutProgressEnd(void **pp, gboolean destroy)
     g_free(*pp);
     return stopped;
 }
-
-
 #endif                          /* USE_GTK */
 
 static void
 TextRolloutProgressStart(const cubeinfo * UNUSED(pci), const int n,
                          rolloutstat UNUSED(aars[2][2]),
-                         rolloutcontext * prc, char asz[][40], gboolean UNUSED(multiple), void **pp)
+                         rolloutcontext * prc, char asz[][FORMATEDMOVESIZE], gboolean UNUSED(multiple), void **pp)
 {
 
     int i;
@@ -989,7 +1022,7 @@ TextRolloutProgressStart(const cubeinfo * UNUSED(pci), const int n,
 }
 
 static int
-TextRolloutProgressEnd(void **pp)
+TextRolloutProgressEnd(void * const *pp)
 {
 
     rolloutprogress *prp = *pp;
@@ -1024,7 +1057,7 @@ TextRolloutProgress(float aarOutput[][NUM_ROLLOUT_OUTPUTS],
         outputl("");
 
     pch = OutputRolloutResult(NULL,
-                              (char (*)[1024]) prp->ppch[iAlternative],
+                              (char (*)[FORMATEDMOVESIZE]) prp->ppch[iAlternative],
                               (float (*)[NUM_ROLLOUT_OUTPUTS]) aarOutput[iAlternative],
                               (float (*)[NUM_ROLLOUT_OUTPUTS]) aarStdDev[iAlternative],
                               &aci[0], iAlternative, 1, prc->fCubeful);
@@ -1072,8 +1105,10 @@ TextRolloutProgress(float aarOutput[][NUM_ROLLOUT_OUTPUTS],
 
     t = time_left(n_games_todo, n_games_done, initial_game_count, prp->tStart);
 
-    outputf(_("Time elapsed %s"), formatDelta(time(NULL) - prp->tStart));
-    outputf(_(" Estimated time left %s\n"), formatDelta(t));
+    outputf(_("Time elapsed"));
+    outputf(" %s ",formatDelta(time(NULL) - prp->tStart));
+    outputf(_("Estimated time left"));
+    outputf(" %s\n", formatDelta(t));
 
     /* estimated SE */
 
@@ -1088,22 +1123,23 @@ TextRolloutProgress(float aarOutput[][NUM_ROLLOUT_OUTPUTS],
         pc = OutputEquityScale(estimatedSE(aarStdDev[0][OUTPUT_EQUITY], iGame, prc->nTrials), &aci[0], &aci[0], FALSE);
 
     if (prp->ppch && prp->ppch[0] && *prp->ppch[0])
-        outputf(_("Estimated SE for \"%s\" after %d trials %s\n"), prp->ppch[0], prc->nTrials, pc);
+        outputf(_("Estimated SE for \"%s\" after %u trials %s\n"), prp->ppch[0], prc->nTrials, pc);
     else
-        outputf(_("Estimated SE after %d trials %s\n"), prc->nTrials, pc);
+        outputf(_("Estimated SE after %u trials %s\n"), prc->nTrials, pc);
 
 }
 
 
 extern void
 RolloutProgressStart(const cubeinfo * pci, const int n,
-                     rolloutstat aars[2][2], rolloutcontext * prc, char asz[][40], gboolean multiple, void **pp)
+                     rolloutstat aars[2][2], rolloutcontext * prc, char asz[][FORMATEDMOVESIZE], gboolean multiple,
+                     void **pp)
 {
 
     if (!fShowProgress)
         return;
 
-#if USE_GTK
+#if defined(USE_GTK)
     if (fX) {
         GTKRolloutProgressStart(pci, n, aars, prc, asz, multiple, pp);
         return;
@@ -1130,7 +1166,7 @@ RolloutProgress(float aarOutput[][NUM_ROLLOUT_OUTPUTS],
     if (!fShowProgress)
         return;
 
-#if USE_GTK
+#if defined(USE_GTK)
     if (fX) {
         GTKRolloutProgress(aarOutput, aarStdDev, prc, aci, initial_game_count, iGame, iAlternative,
                            nRank, rJsd, fStopped, fShowRanks, fCubeRollout, pUserData);
@@ -1150,7 +1186,7 @@ RolloutProgressEnd(void **pp, gboolean destroy)
     if (!fShowProgress)
         return 0;
 
-#if USE_GTK
+#if defined(USE_GTK)
     if (fX) {
         return GTKRolloutProgressEnd(pp, destroy);
     }
