@@ -1,11 +1,11 @@
 /*
- * sgf.c
+ * Copyright (C) 2000-2002 Gary Wong <gtw@gnu.org>
+ * Copyright (C) 2001-2019 the AUTHORS
  *
- * by Gary Wong <gtw@gnu.org>, 2000, 2001.
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of version 3 or later of the GNU General Public License as
- * published by the Free Software Foundation.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,10 +13,9 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
- * $Id: sgf.c,v 1.154 2014/07/27 16:01:11 plm Exp $
+ * $Id: sgf.c,v 1.174 2022/02/26 20:22:15 plm Exp $
  */
 
 #include "config.h"
@@ -43,7 +42,7 @@
 static const char *szFile;
 static int fError;
 
-static int CheckSGFVersion(const char **sz);
+static int CheckSGFVersion(char **sz);
 static void
 ErrorHandler(const char *sz, int UNUSED(fParseError))
 {
@@ -83,8 +82,7 @@ FreeList(listOLD * pl, int nLevel)
         ListDelete(pl->plNext);
     }
 
-    if (nLevel != 4)
-        free(pl);
+    free(pl);
 }
 
 static void
@@ -100,8 +98,6 @@ LoadCollection(char *sz)
 
     listOLD *plCollection, *pl, *plRoot, *plProp;
     FILE *pf;
-    int fBackgammon;
-    property *pp;
 
     fError = FALSE;
     SGFErrorHandler = ErrorHandler;
@@ -127,10 +123,12 @@ LoadCollection(char *sz)
     if (plCollection) {
         pl = plCollection->plNext;
         while (pl != plCollection) {
+            int fBackgammon = FALSE;
+
             plRoot = ((listOLD *) ((listOLD *) pl->p)->plNext->p)->plNext->p;
-            fBackgammon = FALSE;
+
             for (plProp = plRoot->plNext; plProp != plRoot; plProp = plProp->plNext) {
-                pp = plProp->p;
+                property *pp = plProp->p;
 
                 if (pp->ach[0] == 'G' && pp->ach[1] == 'M' && pp->pl->plNext->p && atoi((char *)
                                                                                         pp->pl->plNext->p) == 6) {
@@ -220,72 +218,72 @@ RestoreGS(listOLD * pl, statcontext * psc)
         case 'M':              /* moves */
             psc->fMoves = TRUE;
 
-            psc->anUnforcedMoves[0] = (int)strtol(pch + 2, &pch, 10);
-            psc->anUnforcedMoves[1] = (int)strtol(pch, &pch, 10);
-            psc->anTotalMoves[0] = (int)strtol(pch, &pch, 10);
-            psc->anTotalMoves[1] = (int)strtol(pch, &pch, 10);
+            psc->anUnforcedMoves[0] = (int) strtol(pch + 2, &pch, 10);
+            psc->anUnforcedMoves[1] = (int) strtol(pch, &pch, 10);
+            psc->anTotalMoves[0] = (int) strtol(pch, &pch, 10);
+            psc->anTotalMoves[1] = (int) strtol(pch, &pch, 10);
 
             for (st = SKILL_VERYBAD; st <= SKILL_NONE; st++) {
-                psc->anMoves[0][st] = (int)strtol(pch, &pch, 10);
-                psc->anMoves[1][st] = (int)strtol(pch, &pch, 10);
+                psc->anMoves[0][st] = (int) strtol(pch, &pch, 10);
+                psc->anMoves[1][st] = (int) strtol(pch, &pch, 10);
             }
 
-            psc->arErrorCheckerplay[0][0] = (float)g_ascii_strtod(pch, &pch);
-            psc->arErrorCheckerplay[0][1] = (float)g_ascii_strtod(pch, &pch);
-            psc->arErrorCheckerplay[1][0] = (float)g_ascii_strtod(pch, &pch);
-            psc->arErrorCheckerplay[1][1] = (float)g_ascii_strtod(pch, &pch);
+            psc->arErrorCheckerplay[0][0] = (float) g_ascii_strtod(pch, &pch);
+            psc->arErrorCheckerplay[0][1] = (float) g_ascii_strtod(pch, &pch);
+            psc->arErrorCheckerplay[1][0] = (float) g_ascii_strtod(pch, &pch);
+            psc->arErrorCheckerplay[1][1] = (float) g_ascii_strtod(pch, &pch);
 
             break;
 
         case 'C':              /* cube */
             psc->fCube = TRUE;
 
-            psc->anTotalCube[0] = (int)strtol(pch + 2, &pch, 10);
-            psc->anTotalCube[1] = (int)strtol(pch, &pch, 10);
-            psc->anDouble[0] = (int)strtol(pch, &pch, 10);
-            psc->anDouble[1] = (int)strtol(pch, &pch, 10);
-            psc->anTake[0] = (int)strtol(pch, &pch, 10);
-            psc->anTake[1] = (int)strtol(pch, &pch, 10);
-            psc->anPass[0] = (int)strtol(pch, &pch, 10);
-            psc->anPass[1] = (int)strtol(pch, &pch, 10);
+            psc->anTotalCube[0] = (int) strtol(pch + 2, &pch, 10);
+            psc->anTotalCube[1] = (int) strtol(pch, &pch, 10);
+            psc->anDouble[0] = (int) strtol(pch, &pch, 10);
+            psc->anDouble[1] = (int) strtol(pch, &pch, 10);
+            psc->anTake[0] = (int) strtol(pch, &pch, 10);
+            psc->anTake[1] = (int) strtol(pch, &pch, 10);
+            psc->anPass[0] = (int) strtol(pch, &pch, 10);
+            psc->anPass[1] = (int) strtol(pch, &pch, 10);
 
-            psc->anCubeMissedDoubleDP[0] = (int)strtol(pch, &pch, 10);
-            psc->anCubeMissedDoubleDP[1] = (int)strtol(pch, &pch, 10);
-            psc->anCubeMissedDoubleTG[0] = (int)strtol(pch, &pch, 10);
-            psc->anCubeMissedDoubleTG[1] = (int)strtol(pch, &pch, 10);
-            psc->anCubeWrongDoubleDP[0] = (int)strtol(pch, &pch, 10);
-            psc->anCubeWrongDoubleDP[1] = (int)strtol(pch, &pch, 10);
-            psc->anCubeWrongDoubleTG[0] = (int)strtol(pch, &pch, 10);
-            psc->anCubeWrongDoubleTG[1] = (int)strtol(pch, &pch, 10);
-            psc->anCubeWrongTake[0] = (int)strtol(pch, &pch, 10);
-            psc->anCubeWrongTake[1] = (int)strtol(pch, &pch, 10);
-            psc->anCubeWrongPass[0] = (int)strtol(pch, &pch, 10);
-            psc->anCubeWrongPass[1] = (int)strtol(pch, &pch, 10);
+            psc->anCubeMissedDoubleDP[0] = (int) strtol(pch, &pch, 10);
+            psc->anCubeMissedDoubleDP[1] = (int) strtol(pch, &pch, 10);
+            psc->anCubeMissedDoubleTG[0] = (int) strtol(pch, &pch, 10);
+            psc->anCubeMissedDoubleTG[1] = (int) strtol(pch, &pch, 10);
+            psc->anCubeWrongDoubleDP[0] = (int) strtol(pch, &pch, 10);
+            psc->anCubeWrongDoubleDP[1] = (int) strtol(pch, &pch, 10);
+            psc->anCubeWrongDoubleTG[0] = (int) strtol(pch, &pch, 10);
+            psc->anCubeWrongDoubleTG[1] = (int) strtol(pch, &pch, 10);
+            psc->anCubeWrongTake[0] = (int) strtol(pch, &pch, 10);
+            psc->anCubeWrongTake[1] = (int) strtol(pch, &pch, 10);
+            psc->anCubeWrongPass[0] = (int) strtol(pch, &pch, 10);
+            psc->anCubeWrongPass[1] = (int) strtol(pch, &pch, 10);
 
-            psc->arErrorMissedDoubleDP[0][0] = (float)g_ascii_strtod(pch, &pch);
-            psc->arErrorMissedDoubleDP[0][1] = (float)g_ascii_strtod(pch, &pch);
-            psc->arErrorMissedDoubleTG[0][0] = (float)g_ascii_strtod(pch, &pch);
-            psc->arErrorMissedDoubleTG[0][1] = (float)g_ascii_strtod(pch, &pch);
-            psc->arErrorWrongDoubleDP[0][0] = (float)g_ascii_strtod(pch, &pch);
-            psc->arErrorWrongDoubleDP[0][1] = (float)g_ascii_strtod(pch, &pch);
-            psc->arErrorWrongDoubleTG[0][0] = (float)g_ascii_strtod(pch, &pch);
-            psc->arErrorWrongDoubleTG[0][1] = (float)g_ascii_strtod(pch, &pch);
-            psc->arErrorWrongTake[0][0] = (float)g_ascii_strtod(pch, &pch);
-            psc->arErrorWrongTake[0][1] = (float)g_ascii_strtod(pch, &pch);
-            psc->arErrorWrongPass[0][0] = (float)g_ascii_strtod(pch, &pch);
-            psc->arErrorWrongPass[0][1] = (float)g_ascii_strtod(pch, &pch);
-            psc->arErrorMissedDoubleDP[1][0] = (float)g_ascii_strtod(pch, &pch);
-            psc->arErrorMissedDoubleDP[1][1] = (float)g_ascii_strtod(pch, &pch);
-            psc->arErrorMissedDoubleTG[1][0] = (float)g_ascii_strtod(pch, &pch);
-            psc->arErrorMissedDoubleTG[1][1] = (float)g_ascii_strtod(pch, &pch);
-            psc->arErrorWrongDoubleDP[1][0] = (float)g_ascii_strtod(pch, &pch);
-            psc->arErrorWrongDoubleDP[1][1] = (float)g_ascii_strtod(pch, &pch);
-            psc->arErrorWrongDoubleTG[1][0] = (float)g_ascii_strtod(pch, &pch);
-            psc->arErrorWrongDoubleTG[1][1] = (float)g_ascii_strtod(pch, &pch);
-            psc->arErrorWrongTake[1][0] = (float)g_ascii_strtod(pch, &pch);
-            psc->arErrorWrongTake[1][1] = (float)g_ascii_strtod(pch, &pch);
-            psc->arErrorWrongPass[1][0] = (float)g_ascii_strtod(pch, &pch);
-            psc->arErrorWrongPass[1][1] = (float)g_ascii_strtod(pch, &pch);
+            psc->arErrorMissedDoubleDP[0][0] = (float) g_ascii_strtod(pch, &pch);
+            psc->arErrorMissedDoubleDP[0][1] = (float) g_ascii_strtod(pch, &pch);
+            psc->arErrorMissedDoubleTG[0][0] = (float) g_ascii_strtod(pch, &pch);
+            psc->arErrorMissedDoubleTG[0][1] = (float) g_ascii_strtod(pch, &pch);
+            psc->arErrorWrongDoubleDP[0][0] = (float) g_ascii_strtod(pch, &pch);
+            psc->arErrorWrongDoubleDP[0][1] = (float) g_ascii_strtod(pch, &pch);
+            psc->arErrorWrongDoubleTG[0][0] = (float) g_ascii_strtod(pch, &pch);
+            psc->arErrorWrongDoubleTG[0][1] = (float) g_ascii_strtod(pch, &pch);
+            psc->arErrorWrongTake[0][0] = (float) g_ascii_strtod(pch, &pch);
+            psc->arErrorWrongTake[0][1] = (float) g_ascii_strtod(pch, &pch);
+            psc->arErrorWrongPass[0][0] = (float) g_ascii_strtod(pch, &pch);
+            psc->arErrorWrongPass[0][1] = (float) g_ascii_strtod(pch, &pch);
+            psc->arErrorMissedDoubleDP[1][0] = (float) g_ascii_strtod(pch, &pch);
+            psc->arErrorMissedDoubleDP[1][1] = (float) g_ascii_strtod(pch, &pch);
+            psc->arErrorMissedDoubleTG[1][0] = (float) g_ascii_strtod(pch, &pch);
+            psc->arErrorMissedDoubleTG[1][1] = (float) g_ascii_strtod(pch, &pch);
+            psc->arErrorWrongDoubleDP[1][0] = (float) g_ascii_strtod(pch, &pch);
+            psc->arErrorWrongDoubleDP[1][1] = (float) g_ascii_strtod(pch, &pch);
+            psc->arErrorWrongDoubleTG[1][0] = (float) g_ascii_strtod(pch, &pch);
+            psc->arErrorWrongDoubleTG[1][1] = (float) g_ascii_strtod(pch, &pch);
+            psc->arErrorWrongTake[1][0] = (float) g_ascii_strtod(pch, &pch);
+            psc->arErrorWrongTake[1][1] = (float) g_ascii_strtod(pch, &pch);
+            psc->arErrorWrongPass[1][0] = (float) g_ascii_strtod(pch, &pch);
+            psc->arErrorWrongPass[1][1] = (float) g_ascii_strtod(pch, &pch);
 
             break;
 
@@ -295,14 +293,14 @@ RestoreGS(listOLD * pl, statcontext * psc)
             pch += 2;
 
             for (lt = LUCK_VERYBAD; lt <= LUCK_VERYGOOD; lt++) {
-                psc->anLuck[0][lt] = (int)strtol(pch, &pch, 10);
-                psc->anLuck[1][lt] = (int)strtol(pch, &pch, 10);
+                psc->anLuck[0][lt] = (int) strtol(pch, &pch, 10);
+                psc->anLuck[1][lt] = (int) strtol(pch, &pch, 10);
             }
 
-            psc->arLuck[0][0] = (float)g_ascii_strtod(pch, &pch);
-            psc->arLuck[0][1] = (float)g_ascii_strtod(pch, &pch);
-            psc->arLuck[1][0] = (float)g_ascii_strtod(pch, &pch);
-            psc->arLuck[1][1] = (float)g_ascii_strtod(pch, &pch);
+            psc->arLuck[0][0] = (float) g_ascii_strtod(pch, &pch);
+            psc->arLuck[0][1] = (float) g_ascii_strtod(pch, &pch);
+            psc->arLuck[1][0] = (float) g_ascii_strtod(pch, &pch);
+            psc->arLuck[1][1] = (float) g_ascii_strtod(pch, &pch);
 
             break;
 
@@ -320,7 +318,7 @@ CopyEscapedString(const char *pchOrig)
 
     char *sz, *pch;
 
-    for (pch = sz = malloc(strlen(pchOrig) + 1); *pchOrig; pchOrig++) {
+    for (pch = sz = g_malloc(strlen(pchOrig) + 1); *pchOrig; pchOrig++) {
         if (*pchOrig == '\\') {
             if (pchOrig[1] == '\\') {
                 *pch++ = '\\';
@@ -350,7 +348,7 @@ RestoreText(char *sz, char **ppch)
     if (!sz || !*sz)
         return;
 
-    free(*ppch);
+    g_free(*ppch);
 
     *ppch = CopyEscapedString(sz);
 }
@@ -449,7 +447,7 @@ RestoreRootNode(listOLD * pl)
             if (*++pch != '+')
                 continue;
 
-            pmr->g.nPoints = (int)strtol(pch, &pch, 10);
+            pmr->g.nPoints = (int) strtol(pch, &pch, 10);
             if (pmr->g.nPoints < 1)
                 pmr->g.nPoints = 1;
 
@@ -475,9 +473,9 @@ RestoreRootNode(listOLD * pl)
             RestoreText(pp->pl->plNext->p, &mi.pchRating[1]);
         else if (pp->ach[0] == 'D' && pp->ach[1] == 'T') {
             /* DT - Date */
-            int nYear, nMonth, nDay;
+            unsigned int nYear, nMonth, nDay;
 
-            if (pp->pl->plNext->p && sscanf(pp->pl->plNext->p, "%d-%d-%d", &nYear, &nMonth, &nDay) == 3) {
+            if (pp->pl->plNext->p && sscanf(pp->pl->plNext->p, "%u-%u-%u", &nYear, &nMonth, &nDay) == 3) {
                 mi.nYear = nYear;
                 mi.nMonth = nMonth;
                 mi.nDay = nDay;
@@ -516,17 +514,18 @@ Point(char ch, int f)
 static void
 RestoreRolloutScore(move * pm, const char *sz)
 {
-
     char *pc = strstr(sz, "Score");
 
-    pm->rScore = -99999;
-    pm->rScore2 = -99999;
+    if (pc) {
+        pc += 6;
+        pm->rScore = (float) g_ascii_strtod(pc, &pc);
+        pm->rScore2 = (float) g_ascii_strtod(pc, &pc);
+    } else {
+        pm->rScore = -99999;
+        pm->rScore2 = -99999;
+    }
 
-    if (!pc)
-        return;
-    pc += 6;
-    pm->rScore = (float)g_ascii_strtod(pc, &pc);
-    pm->rScore2 = (float)g_ascii_strtod(pc, &pc);
+    return;
 }
 
 static void
@@ -560,7 +559,7 @@ RestoreRolloutOutput(float ar[NUM_ROLLOUT_OUTPUTS], const char *sz, const char *
     pc += strlen(szKeyword) + 1;
 
     for (i = 0; i < 7; i++)
-        ar[i] = (float)g_ascii_strtod(pc, &pc);
+        ar[i] = (float) g_ascii_strtod(pc, &pc);
 
 }
 
@@ -580,26 +579,22 @@ InitEvalContext(evalcontext * pec)
 static void
 RestoreEvalContext(evalcontext * pec, char *pc)
 {
-
-    int fUsePrune = 0;
-    int red __attribute((unused)) = 0;
-    int ver = CheckSGFVersion((const char **) &pc);
+    int ver = CheckSGFVersion(&pc);
 
     InitEvalContext(pec);
-    pec->nPlies = (unsigned int)strtol(pc, &pc, 10);
+    pec->nPlies = (unsigned int) strtol(pc, &pc, 10);
     if (*pc == 'C') {
         pec->fCubeful = TRUE;
         pc++;
     }
     if (ver < 3) {
-        red = (int)strtol(pc, &pc, 10);
-        pec->fDeterministic = (unsigned int)strtol(pc, &pc, 10);
-        pec->rNoise = (float)g_ascii_strtod(pc, &pc);
+        (void) strtol(pc, &pc, 10);     /* reduced evaluation -- no longer supported */
+        pec->fDeterministic = (unsigned int) strtol(pc, &pc, 10);
+        pec->rNoise = (float) g_ascii_strtod(pc, &pc);
     } else {
-        pec->fDeterministic = (unsigned int)strtol(pc, &pc, 10);
-        pec->rNoise = (float)g_ascii_strtod(pc, &pc);
-        fUsePrune = (unsigned int)strtol(pc, &pc, 10);
-        pec->fUsePrune = fUsePrune;
+        pec->fDeterministic = (unsigned int) strtol(pc, &pc, 10);
+        pec->rNoise = (float) g_ascii_strtod(pc, &pc);
+        pec->fUsePrune = (unsigned int) strtol(pc, &pc, 10);
     }
 }
 
@@ -646,13 +641,13 @@ RestoreRolloutRolloutContext(rolloutcontext * prc, const char *sz)
     prc->fLateEvals = 0;
     prc->fStopOnSTD = 0;
     prc->nLate = 0;
-    prc->nMinimumGames = 144;
+    prc->nMinimumGames = 324;
     prc->rStdLimit = 0.01f;
 
     if (!pc)
         return;
 
-    sscanf(pc, "RC %d %d %d %hu %u \"%[^\"]\" %lu %d %d %d",
+    sscanf(pc, "RC %d %d %d %hu %u \"%1023[^\"]\" %lu %d %d %d",
            &fCubeful,
            &fVarRedn,
            &fInitial, &prc->nTruncate, &prc->nTrials, szTemp, &prc->nSeed, &fRotate, &fTruncBearoff2, &fTruncBearoffOS);
@@ -729,7 +724,7 @@ RestoreRolloutInternals(evalsetup * pes, const char *sz)
 }
 
 static int
-CheckSGFVersion(const char **sz)
+CheckSGFVersion(char **sz)
 {
 
     int n;
@@ -759,9 +754,9 @@ RestoreRolloutMoveFilter(const char *sz, char *name, movefilter mf[MAX_FILTER_PL
 
     pc += strlen(name);
     for (i = 0; i < nPlies; ++i) {
-        mf[nPlies - 1][i].Accept = (int)strtol(pc, &pc, 10);
-        mf[nPlies - 1][i].Extra = (int)strtol(pc, &pc, 10);
-        mf[nPlies - 1][i].Threshold = (float)g_ascii_strtod(pc, &pc);
+        mf[nPlies - 1][i].Accept = (int) strtol(pc, &pc, 10);
+        mf[nPlies - 1][i].Extra = (int) strtol(pc, &pc, 10);
+        mf[nPlies - 1][i].Threshold = (float) g_ascii_strtod(pc, &pc);
     }
 }
 
@@ -779,7 +774,7 @@ RestoreExtendedRolloutContext(rolloutcontext * prc, const char *sz)
     if (!pc)
         return;
 
-    if (sscanf(pc, "RC %d %d %d %d %d %d %hu %d %d %hu \"%[^\"]\" %lu",
+    if (sscanf(pc, "RC %d %d %d %d %d %d %hu %d %d %hu \"%1023[^\"]\" %lu",
                &fCubeful,
                &fVarRedn,
                &fInitial,
@@ -798,7 +793,7 @@ RestoreExtendedRolloutContext(rolloutcontext * prc, const char *sz)
     prc->fTruncBearoff2 = fTruncBearoff2;
     prc->fTruncBearoffOS = fTruncBearoffOS;
     prc->rngRollout = RNG_MERSENNE;
-    prc->nMinimumGames = 144;
+    prc->nMinimumGames = 324;
     prc->rStdLimit = 0.01f;
 
     for (i = 0; i < 2; ++i) {
@@ -823,7 +818,7 @@ RestoreExtendedRolloutContext(rolloutcontext * prc, const char *sz)
 }
 
 static void
-RestoreExtendedCubeRollout(const char *sz,
+RestoreExtendedCubeRollout(char *sz,
                            float aarOutput[][NUM_ROLLOUT_OUTPUTS],
                            float aarStdDev[][NUM_ROLLOUT_OUTPUTS], evalsetup * pes)
 {
@@ -840,7 +835,7 @@ RestoreExtendedCubeRollout(const char *sz,
 }
 
 static void
-RestoreExtendedRollout(move * pm, const char *sz)
+RestoreExtendedRollout(move * pm, char *sz)
 {
 
     evalsetup *pes = &pm->esMove;
@@ -865,10 +860,7 @@ RestoreDoubleAnalysis(property * pp,
 {
 
     char *pch = pp->pl->plNext->p;
-    int nReduced __attribute((unused));
     int fUsePrune = 0;
-    /* leftovers from earlier formats */
-    float arUnused[4] __attribute__ ((unused));
     int ver;
     int i, j;
 
@@ -877,52 +869,51 @@ RestoreDoubleAnalysis(property * pp,
         /* EVAL_EVAL */
         ++pch;
         pes->et = EVAL_EVAL;
-        ver = CheckSGFVersion((const char **) &pch);
-        nReduced = 0;
-        pes->ec.rNoise = 0.0f;
+        ver = CheckSGFVersion(&pch);
         memset(aarOutput[0], 0, NUM_ROLLOUT_OUTPUTS * sizeof(float));
         memset(aarOutput[1], 0, NUM_ROLLOUT_OUTPUTS * sizeof(float));
-        aarOutput[0][OUTPUT_CUBEFUL_EQUITY] = -20000.0;
-        aarOutput[1][OUTPUT_CUBEFUL_EQUITY] = -20000.0;
+        aarOutput[0][OUTPUT_CUBEFUL_EQUITY] = -20000.0f;
+        aarOutput[1][OUTPUT_CUBEFUL_EQUITY] = -20000.0f;
 
         if (ver < 2) {
-            arUnused[0] = (float)g_ascii_strtod(pch, &pch);
-            arUnused[1] = (float)g_ascii_strtod(pch, &pch);
-            arUnused[2] = (float)g_ascii_strtod(pch, &pch);
-            arUnused[3] = (float)g_ascii_strtod(pch, &pch);
-            pes->ec.nPlies = (unsigned int)strtol(pch, &pch, 10);
+            /* skip 4 fields left over from earlier formats */
+            (void) g_ascii_strtod(pch, &pch);
+            (void) g_ascii_strtod(pch, &pch);
+            (void) g_ascii_strtod(pch, &pch);
+            (void) g_ascii_strtod(pch, &pch);
+            pes->ec.nPlies = (unsigned int) strtol(pch, &pch, 10);
             if (*pch == 'C') {
                 pes->ec.fCubeful = TRUE;
                 pch++;
             }
-            nReduced = (int)strtol(pch, &pch, 10);
-            pes->ec.fDeterministic = (unsigned int)strtol(pch, &pch, 10);
-            pes->ec.rNoise = (float)g_ascii_strtod(pch, &pch);
+            (void) strtol(pch, &pch, 10);       /* reduced evaluation -- no longer supported */
+            pes->ec.fDeterministic = (unsigned int) strtol(pch, &pch, 10);
+            pes->ec.rNoise = (float) g_ascii_strtod(pch, &pch);
         } else if (ver == 2) {
-            pes->ec.nPlies = (unsigned int)strtol(pch, &pch, 10);
+            pes->ec.nPlies = (unsigned int) strtol(pch, &pch, 10);
             if (*pch == 'C') {
                 pes->ec.fCubeful = TRUE;
                 pch++;
             }
-            nReduced = (int)strtol(pch, &pch, 10);
-            pes->ec.fDeterministic = (unsigned int)strtol(pch, &pch, 10);
-            pes->ec.rNoise = (float)g_ascii_strtod(pch, &pch);
-            fUsePrune = (int)strtol(pch, &pch, 10);
+            (void) strtol(pch, &pch, 10);       /* reduced evaluation -- no longer supported */
+            pes->ec.fDeterministic = (unsigned int) strtol(pch, &pch, 10);
+            pes->ec.rNoise = (float) g_ascii_strtod(pch, &pch);
+            fUsePrune = (int) strtol(pch, &pch, 10);
         } else {
-            pes->ec.nPlies = (unsigned int)strtol(pch, &pch, 10);
+            pes->ec.nPlies = (unsigned int) strtol(pch, &pch, 10);
             if (*pch == 'C') {
                 pes->ec.fCubeful = TRUE;
                 pch++;
             }
-            pes->ec.fDeterministic = (unsigned int)strtol(pch, &pch, 10);
-            pes->ec.rNoise = (float)g_ascii_strtod(pch, &pch);
-            fUsePrune = (int)strtol(pch, &pch, 10);
+            pes->ec.fDeterministic = (unsigned int) strtol(pch, &pch, 10);
+            pes->ec.rNoise = (float) g_ascii_strtod(pch, &pch);
+            fUsePrune = (int) strtol(pch, &pch, 10);
         }
         pes->ec.fUsePrune = fUsePrune;
 
         for (i = 0; i < 2; i++)
             for (j = 0; j < 7; j++)
-                aarOutput[i][j] = (float)g_ascii_strtod(pch, &pch);
+                aarOutput[i][j] = (float) g_ascii_strtod(pch, &pch);
         break;
 
     case 'R':
@@ -948,7 +939,7 @@ RestoreMarkedMoves(property * pp, movelist * pml)
 {
     int val;
     char *pch = pp->pl->plNext->p;
-    while ((val = (int)strtol(pch, &pch, 10))) {
+    while ((val = (int) strtol(pch, &pch, 10))) {
         if (val > (int) pml->cMoves || val < 1)
             continue;
         pml->amMoves[val - 1].cmark = CMARK_ROLLOUT;
@@ -960,13 +951,8 @@ RestoreMoveAnalysis(property * pp, int fPlayer,
                     movelist * pml, unsigned int *piMove, evalsetup * pesChequer, const matchstate * pms)
 {
     listOLD *pl = pp->pl->plNext;
-    const char *pc;
-    char *pch;
-    char ch;
     move *pm;
     int i;
-    int fDeterministic __attribute((unused));
-    int nReduced __attribute((unused));
     int fUsePrune = 0;
     TanBoard anBoardMove;
     int ver;
@@ -984,11 +970,13 @@ RestoreMoveAnalysis(property * pp, int fPlayer,
     pml->cMaxMoves = pml->cMaxPips = pml->iMoveBest = 0;
     pml->rBestScore = 0;
 
-    pm = pml->amMoves = calloc(pml->cMoves, sizeof(move));
+    pm = pml->amMoves = g_malloc0(pml->cMoves * sizeof(move));
 
     pesChequer->et = EVAL_NONE;
 
     for (pl = pp->pl->plNext->plNext; pl->p; pl = pl->plNext, pm++) {
+        char *pc, *pch, ch;
+
         pc = pl->p;
 
         /* FIXME we could work these out, but it hardly seems worth it */
@@ -1009,51 +997,55 @@ RestoreMoveAnalysis(property * pp, int fPlayer,
         ApplyMove(anBoardMove, pm->anMove, FALSE);
         PositionKey((ConstTanBoard) anBoardMove, &pm->key);
 
-        pch = (char *) pc;
+        pch = pc;
         while (isspace(ch = *pch++)) {
-        };
+        }
         pch++;
 
         switch (ch) {
         case 'E':
-            ver = CheckSGFVersion((const char **) &pch);
+            ver = CheckSGFVersion(&pch);
             /* EVAL_EVAL */
             pm->esMove.et = EVAL_EVAL;
-            nReduced = 0;
-            fDeterministic = 0;
             for (i = 0; i < 5; i++)
-                pm->arEvalMove[i] = (float)g_ascii_strtod(pch, &pch);
-            pm->rScore = (float)g_ascii_strtod(pch, &pch);
+                pm->arEvalMove[i] = (float) g_ascii_strtod(pch, &pch);
+            pm->rScore = (float) g_ascii_strtod(pch, &pch);
             if (ver < 2) {
-                pm->esMove.ec.nPlies = (unsigned int)strtol(pch, &pch, 10);
+                pm->esMove.ec.nPlies = (unsigned int) strtol(pch, &pch, 10);
                 if (*pch == 'C') {
                     pm->esMove.ec.fCubeful = TRUE;
                     pch++;
                 }
-                nReduced = (int)strtol(pch, &pch, 10);
-                pm->esMove.ec.fDeterministic = (unsigned int)strtol(pch, &pch, 10);
-                pm->esMove.ec.rNoise = (float)g_ascii_strtod(pch, &pch);
+                (void) strtol(pch, &pch, 10);   /* reduced evaluation -- no longer supported */
+                pm->esMove.ec.fDeterministic = (unsigned int) strtol(pch, &pch, 10);
+                pm->esMove.ec.rNoise = (float) g_ascii_strtod(pch, &pch);
+/*
+ * Alternatives for ver == 2 and ver > 2 (that is == 3) are identical,
+ * version 3 still writes a reduced evalution flag (set to 0) and
+ * discards it on reads. See comment in WriteMoveAnalysis()
+ */
+#if 0
             } else if (ver == 2) {
-                pm->esMove.ec.nPlies = (unsigned int)strtol(pch, &pch, 10);
+                pm->esMove.ec.nPlies = (unsigned int) strtol(pch, &pch, 10);
                 if (*pch == 'C') {
                     pm->esMove.ec.fCubeful = TRUE;
                     pch++;
                 }
-                nReduced = (int)strtol(pch, &pch, 10);
-                pm->esMove.ec.fDeterministic = (unsigned int)strtol(pch, &pch, 10);
-                pm->esMove.ec.rNoise = (float)g_ascii_strtod(pch, &pch);
-                fUsePrune = (int)strtol(pch, &pch, 10);
+                (void) strtol(pch, &pch, 10);   /* reduced evaluation -- no longer supported */
+                pm->esMove.ec.fDeterministic = (unsigned int) strtol(pch, &pch, 10);
+                pm->esMove.ec.rNoise = (float) g_ascii_strtod(pch, &pch);
+                fUsePrune = (int) strtol(pch, &pch, 10);
+#endif
             } else {
-                pm->esMove.ec.nPlies = (unsigned int)strtol(pch, &pch, 10);
+                pm->esMove.ec.nPlies = (unsigned int) strtol(pch, &pch, 10);
                 if (*pch == 'C') {
                     pm->esMove.ec.fCubeful = TRUE;
                     pch++;
                 }
-                /* FIXME: reduced shouldn't be saved/loaded */
-                nReduced = (int)strtol(pch, &pch, 10);
-                pm->esMove.ec.fDeterministic = (unsigned int)strtol(pch, &pch, 10);
-                pm->esMove.ec.rNoise = (float)g_ascii_strtod(pch, &pch);
-                fUsePrune = (int)strtol(pch, &pch, 10);
+                (void) strtol(pch, &pch, 10);   /* reduced evaluation -- no longer supported */
+                pm->esMove.ec.fDeterministic = (unsigned int) strtol(pch, &pch, 10);
+                pm->esMove.ec.rNoise = (float) g_ascii_strtod(pch, &pch);
+                fUsePrune = (int) strtol(pch, &pch, 10);
             }
             pm->esMove.ec.fUsePrune = fUsePrune;
             break;
@@ -1076,7 +1068,6 @@ RestoreMoveAnalysis(property * pp, int fPlayer,
         if (cmp_evalsetup(pesChequer, &pm->esMove) < 0)
             memcpy(pesChequer, &pm->esMove, sizeof(evalsetup));
 
-
     }
 }
 
@@ -1085,13 +1076,13 @@ PointList(listOLD * pl, int an[])
 {
 
     int i;
-    char *pch, ch0, ch1;
+    char ch0, ch1;
 
     for (i = 0; i < 25; i++)
         an[i] = 0;
 
     for (; pl->p; pl = pl->plNext) {
-        pch = pl->p;
+        char *pch = pl->p;
 
         if (strchr(pch, ':')) {
             ch0 = ch1 = 0;
@@ -1174,7 +1165,7 @@ RestoreNode(listOLD * pl)
 
                 if (pmr->anDice[0] < 1 || pmr->anDice[0] > 6 || pmr->anDice[1] < 1 || pmr->anDice[1] > 6) {
                     /* illegal roll -- ignore */
-                    free(pmr);
+                    g_free(pmr);
                     pmr = NULL;
                 }
             }
@@ -1227,7 +1218,7 @@ RestoreNode(listOLD * pl)
                 pmr->scp.fCubeOwner = 0;
                 break;
             default:
-                free(pmr);
+                g_free(pmr);
                 pmr = NULL;
             }
 
@@ -1274,7 +1265,7 @@ RestoreNode(listOLD * pl)
         else if (pp->ach[0] == 'D' && pp->ach[1] == 'C')
             ast[1] = SKILL_DOUBTFUL;
         else if (pp->ach[0] == 'L' && pp->ach[1] == 'U')
-            rLuck = (float)g_ascii_strtod(pp->pl->plNext->p, NULL);
+            rLuck = (float) g_ascii_strtod(pp->pl->plNext->p, NULL);
         else if (pp->ach[0] == 'G' && pp->ach[1] == 'B')
             /* good for black */
             lt = *((char *) pp->pl->plNext->p) == '2' ? LUCK_VERYGOOD : LUCK_GOOD;
@@ -1375,7 +1366,7 @@ static void
 RestoreGame(listOLD * pl)
 {
 
-    moverecord *pmr, *pmrResign;
+    moverecord *pmr;
 
     InitBoard(ms.anBoard, ms.bgv);
 
@@ -1403,7 +1394,7 @@ RestoreGame(listOLD * pl)
          * inverted when shown. /jth 2003-10-12 
          * ms.fTurn = ms.fMove = -1; */
 
-        pmrResign = NewMoveRecord();
+        moverecord *pmrResign = NewMoveRecord();
 
         pmrResign->mt = MOVE_RESIGN;
 
@@ -1523,7 +1514,6 @@ CommandLoadPosition(char *sz)
 extern void
 CommandLoadMatch(char *sz)
 {
-
     listOLD *pl;
 
     sz = NextToken(&sz);
@@ -1534,6 +1524,8 @@ CommandLoadMatch(char *sz)
     }
 
     if ((pl = LoadCollection(sz))) {
+        int nGames = 0, nMoves = 0;
+
         /* FIXME make sure the root nodes have MI properties; if not,
          * we're loading a session. */
         if (!get_input_discard())
@@ -1550,6 +1542,7 @@ CommandLoadMatch(char *sz)
 
         for (pl = pl->plNext; pl->p; pl = pl->plNext) {
             RestoreGame(pl->p);
+            nGames++;
         }
 
         FreeGameTreeSeq(pl);
@@ -1565,8 +1558,35 @@ CommandLoadMatch(char *sz)
 
         setDefaultFileName(sz);
 
-        /* FIXME : don't do this if we just loaded a single position */
-        if (fGotoFirstGame)
+        for (pl = plGame->plNext; pl != plGame; pl = pl->plNext) {
+            moverecord *pmr = pl->p;
+
+            switch (pmr->mt) {
+            case MOVE_NORMAL:
+            case MOVE_DOUBLE:
+            case MOVE_TAKE:
+            case MOVE_DROP:
+                nMoves++;
+                break;
+            default:
+                /* do not count the other pseudo-moves */
+                break;
+            }
+        }
+
+        if (nGames == 1 && nMoves == 1) {
+            moverecord *pmr;
+
+            CommandFirstMove(NULL);
+            pl = plGame->plNext;
+            pmr = pl->p;
+            while (pmr->mt != MOVE_NORMAL && pmr->mt != MOVE_DOUBLE) {
+                CommandNext(NULL);
+                pl = pl->plNext;
+                pmr = pl->p;
+            }
+            CommandPrevious(NULL);
+        } else if (fGotoFirstGame)
             CommandFirstGame(NULL);
 
     }
@@ -1578,7 +1598,7 @@ WriteEscapedString(FILE * pf, char *pch, int fEscapeColons)
 
     char *sz, *pc;
 
-    sz = (char *) malloc(2 * strlen(pch) + 1);
+    sz = (char *) g_malloc(2 * strlen(pch) + 1);
 
     for (pc = sz; *pch; pch++)
         switch (*pch) {
@@ -1600,10 +1620,10 @@ WriteEscapedString(FILE * pf, char *pch, int fEscapeColons)
             break;
         }
 
-    *pc++ = 0;
+    *pc = 0;
 
     fputs(sz, pf);
-    free(sz);
+    g_free(sz);
 }
 
 static void
@@ -1628,12 +1648,12 @@ WriteMoveFilters(FILE * pf, movefilter mf[MAX_FILTER_PLIES][MAX_FILTER_PLIES], i
 }
 
 static void
-WriteRolloutContext(FILE * pf, const rolloutcontext * prc)
+WriteRolloutContext(FILE * pf, rolloutcontext * prc)
 {
 
     int i;
 
-    fprintf(pf, "RC %u %u %u %u %u %u %d %u %u %d \"%s\" %lu ",
+    fprintf(pf, "RC %u %u %u %u %u %u %u %u %u %u \"%s\" %lu ",
             prc->fCubeful,
             prc->fVarRedn,
             prc->fInitial,
@@ -1651,7 +1671,7 @@ WriteRolloutContext(FILE * pf, const rolloutcontext * prc)
         WriteEvalContext(pf, &prc->aecChequer[i]);
         if (prc->aecChequer[i].nPlies) {
             fprintf(pf, " filt%d ", i);
-            WriteMoveFilters(pf, (movefilter(*)[4]) prc->aaamfChequer[i], prc->aecChequer[i].nPlies);
+            WriteMoveFilters(pf, prc->aaamfChequer[i], prc->aecChequer[i].nPlies);
         }
     }
 
@@ -1664,7 +1684,7 @@ WriteRolloutContext(FILE * pf, const rolloutcontext * prc)
         WriteEvalContext(pf, &prc->aecChequerLate[i]);
         if (prc->aecChequerLate[i].nPlies) {
             fprintf(pf, " latefilt%d ", i);
-            WriteMoveFilters(pf, (movefilter(*)[4]) prc->aaamfLate[i], prc->aecChequerLate[i].nPlies);
+            WriteMoveFilters(pf, prc->aaamfLate[i], prc->aecChequerLate[i].nPlies);
         }
     }
 
@@ -1832,7 +1852,13 @@ WriteMoveAnalysis(FILE * pf, int fPlayer, movelist * pml, unsigned int iMove)
             fprintf(pf, "%u%s %d %u %s %u",
                     pml->amMoves[i].esMove.ec.nPlies,
                     pml->amMoves[i].esMove.ec.fCubeful ? "C" : "",
-                    0, pml->amMoves[i].esMove.ec.fDeterministic, buffer, pml->amMoves[i].esMove.ec.fUsePrune);
+                    /*
+                     * Former reduced evaluation flag
+                     * Kept for backward compatibility ?
+                     * Always 0 in SGF ver. 3
+                     */
+                    0,
+                    pml->amMoves[i].esMove.ec.fDeterministic, buffer, pml->amMoves[i].esMove.ec.fUsePrune);
             break;
 
         case EVAL_ROLLOUT:
@@ -1853,7 +1879,7 @@ WriteMoveAnalysis(FILE * pf, int fPlayer, movelist * pml, unsigned int iMove)
     fprintf(pf, "MR[");
     for (i = 0; i < pml->cMoves; i++) {
         if (pml->amMoves[i].cmark)
-            fprintf(pf, "%d ", i + 1);
+            fprintf(pf, "%u ", i + 1);
     }
     fputc(']', pf);
 }
@@ -2091,13 +2117,12 @@ SaveGame(FILE * pf, listOLD * plGame)
     putc(']', pf);
 
     if (!pmr->g.i) {
-        char szDate[32];
-
         WriteProperty(pf, "WR", mi.pchRating[0]);
         WriteProperty(pf, "BR", mi.pchRating[1]);
         if (mi.nYear) {
-            sprintf(szDate, "%04u-%02u-%02u", mi.nYear, mi.nMonth, mi.nDay);
+            char *szDate = g_strdup_printf("%04u-%02u-%02u", mi.nYear, mi.nMonth, mi.nDay);
             WriteProperty(pf, "DT", szDate);
+            g_free(szDate);
         }
         WriteProperty(pf, "EV", mi.pchEvent);
         WriteProperty(pf, "RO", mi.pchRound);
@@ -2108,9 +2133,6 @@ SaveGame(FILE * pf, listOLD * plGame)
 
     if (pmr->g.fCrawford || pmr->g.fJacoby || pmr->g.bgv != VARIATION_STANDARD || !pmr->g.fCubeUse) {
 
-        static const char *aszSGFVariation[NUM_VARIATIONS] = { NULL, "Nackgammon", "Hypergammon1", "Hypergammon2",
-            "Hypergammon3"
-        };
         int fFirst = TRUE;
 
         fputs("RU[", pf);
@@ -2123,9 +2145,12 @@ SaveGame(FILE * pf, listOLD * plGame)
             AddRule(pf, "CrawfordGame", &fFirst);
         if (pmr->g.fJacoby)
             AddRule(pf, "Jacoby", &fFirst);
-        if (pmr->g.bgv != VARIATION_STANDARD)
-            AddRule(pf, aszSGFVariation[pmr->g.bgv], &fFirst);
+        if (pmr->g.bgv != VARIATION_STANDARD) {
+            static const char *aszSGFVariation[NUM_VARIATIONS] =
+                { NULL, "Nackgammon", "Hypergammon1", "Hypergammon2", "Hypergammon3" };
 
+            AddRule(pf, aszSGFVariation[pmr->g.bgv], &fFirst);
+        }
         fputs("]", pf);
     }
 
@@ -2243,7 +2268,7 @@ SaveGame(FILE * pf, listOLD * plGame)
                 fputs("AW", pf);
                 for (i = 0; i < 25; i++)
                     for (j = 0; j < anBoard[1][i]; j++)
-                        fprintf(pf, "[%c]", 'a' + i);
+                        fprintf(pf, "[%c]", (char) ('a' + i));
             }
 
             for (i = 0, j = 0; i < 25; ++i)
@@ -2253,7 +2278,7 @@ SaveGame(FILE * pf, listOLD * plGame)
                 fputs("AB", pf);
                 for (i = 0; i < 25; i++)
                     for (j = 0; j < anBoard[0][i]; j++)
-                        fprintf(pf, "[%c]", i == 24 ? 'y' : 'x' - i);
+                        fprintf(pf, "[%c]", i == 24 ? 'y' : (char) ('x' - i));
             }
 
             break;
@@ -2337,6 +2362,7 @@ CommandSaveMatch(char *sz)
 
     FILE *pf;
     listOLD *pl;
+    int fDontClose = FALSE;
 
     sz = NextToken(&sz);
 
@@ -2355,9 +2381,10 @@ CommandSaveMatch(char *sz)
     if (!confirmOverwrite(sz, fConfirmSave))
         return;
 
-    if (!strcmp(sz, "-"))
+   if (!strcmp(sz, "-")) {
         pf = stdout;
-    else if (!(pf = g_fopen(sz, "w"))) {
+        fDontClose = TRUE;
+    } else if (!(pf = g_fopen(sz, "w"))) {
         outputerr(sz);
         return;
     }
@@ -2365,7 +2392,7 @@ CommandSaveMatch(char *sz)
     for (pl = lMatch.plNext; pl != &lMatch; pl = pl->plNext)
         SaveGame(pf, pl->p);
 
-    if (pf != stdout)
+    if (!fDontClose)
         fclose(pf);
 
     setDefaultFileName(sz);
@@ -2385,6 +2412,7 @@ CommandSavePosition(char *sz)
     moverecord *pmr_cur = NULL;
     moverecord *pmscv;
     moverecord *pmscp;
+    int fDontClose = FALSE;
 
     sz = NextToken(&sz);
 
@@ -2401,9 +2429,10 @@ CommandSavePosition(char *sz)
     if (!confirmOverwrite(sz, fConfirmSave))
         return;
 
-    if (!strcmp(sz, "-"))
+    if (!strcmp(sz, "-")) {
         pf = stdout;
-    else if (!(pf = g_fopen(sz, "w"))) {
+        fDontClose = TRUE;
+    } else if (!(pf = g_fopen(sz, "w"))) {
         outputerr(sz);
         return;
     }
@@ -2484,17 +2513,17 @@ CommandSavePosition(char *sz)
 
     SaveGame(pf, &l);
 
-    if (pf != stdout)
+    if (!fDontClose)
         fclose(pf);
 
     while (l.plNext->p)
         ListDelete(l.plNext);
 
-    free(pmgi);
-    free(pmsb);
-    free(pmsd);
-    free(pmscv);
-    free(pmscp);
+    g_free(pmgi);
+    g_free(pmsb);
+    g_free(pmsd);
+    g_free(pmscv);
+    g_free(pmscp);
 
     setDefaultFileName(sz);
 }
